/*:VRX         Main
*/
/*  Main
*/
ok = VRRedirectStdIO('OFF')
Main:
/*  Process the arguments.
    Get the parent window.
*/
    parse source . calledAs .
    parent = ""
    argCount = arg()
    argOff = 0
    if( calledAs \= "COMMAND" )then do
        if argCount >= 1 then do
            parent = arg(1)
            argCount = argCount - 1
            argOff = 1
        end
    end; else do
        call VROptions 'ImplicitNames'
        call VROptions 'NoEchoQuit'
    end
    InitArgs.0 = argCount
    if( argCount > 0 )then do i = 1 to argCount
        InitArgs.i = arg( i + argOff )
    end
    drop calledAs argCount argOff

/*  Load the windows
*/
    call VRInit
    parse source . . spec
    _VREPrimaryWindowPath = ,
        VRParseFileName( spec, "dpn" ) || ".VRW"
    _VREPrimaryWindow = ,
        VRLoad( parent, _VREPrimaryWindowPath )
    drop parent spec
    if( _VREPrimaryWindow == "" )then do
        call VRMessage "", "Cannot load window:" VRError(), ,
            "Error!"
        _VREReturnValue = 32000
        signal _VRELeaveMain
    end

/*  Process events
*/
    call Init
    signal on halt
    do while( \ VRGet( _VREPrimaryWindow, "Shutdown" ) )
        _VREEvent = VREvent()
        interpret _VREEvent
    end
_VREHalt:
    _VREReturnValue = Fini()
    call VRDestroy _VREPrimaryWindow
_VRELeaveMain:
    call VRFini
exit _VREReturnValue

VRLoadSecondary:
    __vrlsWait = abbrev( 'WAIT', translate(arg(2)), 1 )
    if __vrlsWait then do
        call VRFlush
    end
    __vrlsHWnd = VRLoad( VRWindow(), VRWindowPath(), arg(1) )
    if __vrlsHWnd = '' then signal __vrlsDone
    if __vrlsWait \= 1 then signal __vrlsDone
    call VRSet __vrlsHWnd, 'WindowMode', 'Modal' 
    __vrlsTmp = __vrlsWindows.0
    if( DataType(__vrlsTmp) \= 'NUM' ) then do
        __vrlsTmp = 1
    end
    else do
        __vrlsTmp = __vrlsTmp + 1
    end
    __vrlsWindows.__vrlsTmp = VRWindow( __vrlsHWnd )
    __vrlsWindows.0 = __vrlsTmp
    do while( VRIsValidObject( VRWindow() ) = 1 )
        __vrlsEvent = VREvent()
        interpret __vrlsEvent
    end
    __vrlsTmp = __vrlsWindows.0
    __vrlsWindows.0 = __vrlsTmp - 1
    call VRWindow __vrlsWindows.__vrlsTmp 
    __vrlsHWnd = ''
__vrlsDone:
return __vrlsHWnd

/*:VRX         _AnalyseConfigSys
*/
_AnalyseConfigSys: 
    /* From CONFIG.SYS we get the information, if
       XF86SUP.SYS is installed
       WIN32K.SYS is installed (currently)
       which display driver is installed
     */
    ConfigSys = SysBootDrive()'\CONFIG.SYS'
    do while lines(ConfigSys) > 0 
        ConfigLine = translate(strip(Linein(ConfigSys)))
        if left(ConfigLine,3) <> 'REM' then do
            select
                when Pos('XF86SUP.SYS', ConfigLine) > 0 then Foundxf86supsys = 1
                when Pos('WIN32K.SYS', ConfigLine) > 0 then do
                    Win32Kparm = substr(ConfigLine,pos("WIN32K.SYS", ConfigLine) + 11)
                    FoundWin32KSysNow = 1
                end
                when Pos('SDDGRADD', ConfigLine) > 0 then  do
                    Driver = "Scitech Display Doctor"
                    FoundScitech = 1
                end
                when Pos('MGAX64', ConfigLine) > 0 then      Driver = "Matrox Driver"
                when Pos('GENGRADD', ConfigLine) > 0 then    Driver = "Generic GRADD"
                when Pos('ATIGRADD', ConfigLine) > 0 then    Driver = "ATI GRADD"
                when Pos('CHPGRADD', ConfigLine) > 0 then    Driver = "CHP GRADD"
                when Pos('DYNGRADD', ConfigLine) > 0 then    Driver = "DYN GRADD"
                when Pos('G_VGRADD', ConfigLine) > 0 then    Driver = "G_V GRADD"
                when Pos('MGAGRADD', ConfigLine) > 0 then    Driver = "Matrox GRADD"
                when Pos('NMGRADD', ConfigLine) > 0 then     Driver = "NeoMagic GRADD"
                when Pos('S3GRADD', ConfigLine) > 0 then     Driver = "S3 GRADD"
                when Pos('TRIGRADD', ConfigLine) > 0 then    Driver = "Trident GRADD"
                when Pos('VIRGEGRADD', ConfigLine) > 0 then  Driver = "S3 ViRGE GRADD"
                when Pos('VGAGRADD', ConfigLine) > 0 then    Driver = "VGA GRADD"
                /* add other display drivers here */
                otherwise nop
            end
        end
    end 
    call lineout(ConfigSys)
return
/*:VRX         _AnalyseKernel32DLL
*/
_AnalyseKernel32DLL: 
    /*  we get build level information
        from Kernel32.DLL by spawning
        BLDLEVEL.EXE, redirecting the 
        output and analysing it */
        
    /* return if we do not have a valid directory */
    if OdinSysDir = '' then return
    
    /* get Kernel32 version */
    BldInfo     = 'BldInfo.TXT'
    
    'BLDLevel 'OdinSysDir'Kernel32.DLL >'BldInfo
    do until lines(BldInfo) = 0
        BldLine = translate(linein(BldInfo))
        if pos('FILE VERSION',BldLine) > 0 then do
            Kernel32Ver = word(BldLine,3)
        end
        if pos('DATE/TIME',BldLine) > 0 then do
            Kernel32Date = word(BldLine,4)||right('0'||_xmonth(word(BldLine,3)),2)||right('0'||word(BldLine,2),2)
        end
    end
    call lineout BldInfo
    ok = SysFileDelete(BldInfo)
return

/*:VRX         _AnalyseOdin32_xLog
*/
_AnalyseOdin32_xLog: 
    /* We get the information about
       WIN32K.SYS installed (when bug occured)
       Odin directory
       Odin\System directory
       Commandline of Win32 program
       location of Odin.INI
       we have a visible progressbar,
       this analysis is usually done after 
       reading 50 lines or so of the logfile
     */
    If LogFileName = '' then return

    /* clear variables */
    OdinWinDir = ''
    OdinSysDir = ''
    ExeFile    = ''
    FoundWinDir = 0
    FoundSysDir = 0
    FoundExe = 0
    FoundOdinIni = 0

    ExceptFilename = filespec('D',LogFileName)||filespec('P',LogFileName)||'Except.$$$'

    LogFileSize = stream(LogfileName,'c','QUERY SIZE')
    ApproxLines = LogFileSize % 41
    call _InitProgress
    LogLine = ''
    do J = 1 to 5 until FoundWin32KSysAtBugTime = 1
        LogLine = linein(LogFileName)
        if pos('WIN32K - INSTALLED',translate(LogLine)) > 0 then do
            FoundWin32KSysAtBugTime = 1
            Win32KLogLine = LogLine
        end            
    end
    do until lines(LogFileName) = 0
        LogLine = linein(LogFileName)
        select
            when pos('Windows  dir:',LogLine) > 0 then do
                OdinWinDir = strip(Substr(LogLine,pos('Windows  dir:',LogLine)+14))'\'
                FoundWinDir = 1
            end
            when pos('System32 dir:',LogLine) > 0 then do
                OdinSysDir = strip(Substr(LogLine,pos('System32 dir:',LogLine)+14))'\'
                FoundSysDir = 1
            end
            when pos('Cmd line:',LogLine) > 0 then do
                EXEFile = strip(Substr(LogLine,pos('Cmd line:',LogLine)+10))
                FoundExe = 1
            end
            when pos('GetCommandLineA:',LogLine) > 0 then do
                ExeFile = strip(Substr(LogLine,pos('GetCommandLineA:',LogLine)+16))
                FoundExe = 1
            end
            when pos('Odin ini loaded:',LogLine) > 0 then do
                OdinINI = strip(Substr(LogLine,pos('Odin ini loaded:',LogLine)+16))
                FoundOdinIni = 1
            end
            otherwise do
                I = I + 1
                if I // 1000 = 0 then do
                    call _CheckForEvent
                    Progress = I * 100 % ApproxLines
                    ok = VRSet("SL_Progress","Percentile",Progress)
                end
            end
            if FoundWinDir & FoundSysDir & FoundExe & FoundOdinIni then leave
        end
    end
    ok = VRSet("Progress","Visible", 0)
    call lineout(LogFileName)
return
/*:VRX         _AnalyseOdinIni
*/
_AnalyseOdinIni: 
    /* we get the INSTALLDATE information from Odin.INI */
    
    /* old method to locate Odin.INI - obsolete,
       now we get Odin.Ini from the Logfile
    Odin_INI = value('ODIN_INI',,'OS2ENVIRONMENT') */

    do until lines(OdinINI) = 0
        OdinIniLine = translate(linein(OdinINI))
        if pos('INSTALLDATE',OdinIniLine) > 0 then do
            OdinInstLine  = substr(OdinIniLine,pos('INSTALLDATE',OdinIniLine)+12)
            OdinInstDoW   = word(OdinInstLine,1)
            OdinInstMonth = _xmonth(word(OdinInstLine,2))
            OdinInstDay   = right('0'||word(OdinInstLine,3),2)
            OdinInstTime  = word(OdinInstLine,4)
            OdinInstYear  = word(OdinInstLine,5)
            OdinInstDate  = OdinInstDay'.'OdinInstMonth'.'OdinInstYear
            OdinInstDate2 = OdinInstYear||right('0'||OdinInstMonth,2)||right('0'||odinInstday,2)
            leave
        end 
    end
    call lineout(OdinIni)
    LastOdinInst=OdinInstDate' at 'OdinInstTime', current version 'OdinInstVer
    ok = VRSet("EF_LastOdinInst","Value",LastOdinInst)
    if OdinInstDate2 < builddate & builddate <> '' then do
        Msg.Text = "You did not run OdinInst.EXE after your last installation. Run OdinInst.EXE, and rerun your program before submitting an Odin Bug Report!"
        Msg.Type = "Error"
        call _ShowMsg
        ok = VRSet("PB_CreateReport","Visible", 0)
    end
return

/*:VRX         _AnalyseOdininst
*/
_AnalyseOdininst: 
    /*  we get build level information
        from OdinInst.EXE by spawning
        BLDLEVEL.EXE, redirecting the 
        output and analysing it */

    /* get OdinInst Version */
    BldInfo     = 'BldInfo.TXT'
    
    'BLDLevel 'OdinSysDir'OdinInst.exe >'BldInfo
    do until lines(BldInfo) = 0
        BldLine = translate(linein(BldInfo))
        if pos('FILE VERSION',BldLine) > 0 then do
            OdinInstVer = word(BldLine,3)
            leave
        end
    end
    call lineout BldInfo
    ok = SysFileDelete(BldInfo)
return

/*:VRX         _AnalyseScitechDD
*/
_AnalyseScitechDD: 
    /* we know that we have have a Scitech Driver
       now lets determine which one. We analyse
       SDDHELP.SYS and GRAPHICS.LOG */
       
    SDDHelpSys = SysBootDrive()'\OS2\SDDHELP.SYS'

    LogLine = ''
    do until lines(SDDHelpSys) = 0
        LogLine = translate(linein(SDDHelpSys),' ', d2c(0))
        if pos('build',logline) > 0 then do
            SddBuildNr = substr(LogLine,pos('build',logline),10)
        end
    end
    ok = stream(SDDHelpSys,'c','CLOSE')

    GraphicsLog = SysBootDrive()'\OS2\DRIVERS\NUCLEUS\CONFIG\GRAPHICS.LOG'
    License = linein(GraphicsLog)
    select 
        when word(License,1)= 'Valid' & word(License,2) = 'IHV' then Driver = 'Scitech Display Doctor/SE'
        when word(License,1)= 'Registered' & word(License,2) = 'end' then Driver = 'Scitech Display Doctor/Pro'
        otherwise Driver = 'Scitech DD/Demo'
    end

    LogLine = ''
    do until pos('Chipset...',LogLine) > 0 | lines(GraphicsLog) = 0
        LogLine = linein(GraphicsLog)
    end
    ChipSet = ''
    if lines(graphicslog) = 0 then ChipSet = "Detection failed!"
    else do I = 2 to words(logline)
        ChipSet = Chipset||word(Logline,I)||' '
    end
    ok = stream(GraphicsLog,'c','CLOSE')
return

/*:VRX         _AnalyseSyslevelFPK
*/
_AnalyseSyslevelFPK: 
    sys_os2 = SysSearchPath('path', 'syslevel.FPK')
    if sys_os2 = '' then do
        fpk_level = '(none)'
        return 
    end
    sys_info = translate(Charin(sys_os2, 1, 150),' ','00'x)
    Parse Var sys_info . 45 fpk_level 141 os_comp 150
    fpk_level = left(fpk_level,7)
    ok = stream(sys_os2,'c','CLOSE')
return

/*:VRX         _AnalyseSysLevelOS2
*/
_AnalyseSysLevelOS2: 
    sys_os2 = SysSearchPath('path', 'syslevel.os2')
    sys_info = Charin(sys_os2, 1, 150)
    Parse Var sys_info . 61 os_name 141 os_comp 150
    os_name = Strip(os_name, 't', '00'x)
    os_ver  = SysOS2Ver()
    if os_ver = 2.30 then OS2Type = "OS/2 Warp 3 (not supported)"
    select 
        when os_comp = '5639A6100' then do
            if os_ver = 2.40 then OS2Type = "OS/2 Warp 4"
            if os_ver = 2.45 then OS2Type = "OS/2 Warp 4.5"
        end
        when os_comp = '5639A6101' then OS2Type = "OS/2 4.51 MCP or eComStation"
        when os_comp = '5639A6150' then OS2Type = "OS/2 WSeB uni"
        when os_comp = '5639A5550' then OS2Type = "OS/2 WSeB smp"
        otherwise do
        end
    end
    ok = stream(sys_os2,'c','CLOSE')

    /* get Kernel Version */
    BldInfo     = 'BldInfo.TXT'
    'BLDLevel 'SysBootDrive()'\os2krnl >'BldInfo
    do until lines(BldInfo) = 0
        BldLine = linein(BldInfo)
        if pos('Signature',BldLine) > 0 then do
            KrnlBuildVer = translate(word(BldLine,2),'   ','#@:')
            KrnlBuildVer = word(KrnlBuildVer,2)||word(KrnlBuildVer,3)
            leave
        end
    end
    call lineout BldInfo
    ok = SysFileDelete(BldInfo)
return

/*:VRX         _AnalyseWarpINdatabase
*/
_AnalyseWarpINdatabase: 
    FoundWarpIN = 1

    WarpINDir = strip(VRGetIni( "WarpIN", "Path", "User" ),,d2c(0))
    if WarpINDir = '' then do
        Msg.Text = "WarpIN directory not found!"
        Msg.Type = "Warning"
        call _ShowMsg
        FoundWarpIN = 0
        return
    end   
    BDL = left(SysBootDrive(),1)
    ok = SysFileTree(WarpInDir||'\DATBAS_'BDL'.INI',WarpINDB., 'FO')
    select
        when WarpINDB.0 = 0 then do
            Msg.Text = "No WarpIN database file found!"
            Msg.Type = "Warning"
            call _ShowMsg
            FoundWarpIN = 0
        end
        when WarpINDB.0 = 1 then do
            WarpINDB = WarpINDB.1
        end
/*      when WarpINDB.0 > 1 then do
            Msg.Text = "You appear to have more than one WarpIN database! Choosing biggest!"
            Msg.Type = "Warning"
            call _ShowMsg
            MaxSize = 0
            do I = 1 to WarpINDB.0
                CurrSize = query(WarpinDB.I,'c','query size')
                if  CurrSize >= MaxSize then do
                    WarpINDB = WarpINDB.I
                    MaxSize = CurrSize
                end
            end
        end   */
        otherwise do
            msg.Text = 'Something weird has happened! You should not see this message!'
            msg.Type = "Error"
            call _ShowMsg
            FoundWarpIN = 0
        end
    end

/*  WarpINDB = WarpInDir||'\DATBAS_E.INI' /* '\DATBAS_D.INI' */ */

    if FoundWarpIN then do
        ok = SysIni(WarpInDB,'All:',"Apps")
        if apps.0 = 0 then do
            Msg.Text = "WarpIN database "WarpINDB" reported no applications!"
            Msg.Type = "Warning"
            call _ShowMsg
            FoundWarpIN = 0
            return
        end

        ok = VRset("EF_LastWarpIN","HintText","Last time Odin was installed using WarpIN due to "WarpINDB)
        OdinCoreDateTime = ''
        OdinSYsDateTime = ''
        do I = 1 to apps.0
          select
            when pos('Odin System',apps.I) > 0  then do
                OdinSysDateTime = c2x(strip(VRGetIni( apps.I, "InstallDateTime", WarpINDB ),,d2c(0)))
                OdinSysVer = apps.I
            end
            when pos('Odin Core',apps.I) > 0  then do
                OdinCoreDateTime = c2x(strip(VRGetIni( apps.I, "InstallDateTime", WarpINDB ),,d2c(0)))
                OdinCoreVer = apps.I
            end
            otherwise nop
          end
        end
        if OdinCoreDateTime = '' | OdinSysDateTime = ''  then do
            Msg.Text = 'Could not find last WarpIN install info!'
            Msg.Type = "Warning"
            call _ShowMsg
            LastWarpIN = 'Maybe never!'
        end
        else do
            OdinCoretime = _xtime(substr(OdinCoreDateTime,1,6))
            OdinSysTime  = _xtime(substr(OdinSYsDateTime,1,6))
            OdinCoreDate = _xdate(substr(OdinCoreDateTime,9,8))
            OdinSysDate  = _xdate(substr(OdinSYsDateTime,9,8))
            OdinCoreVer  = translate(substr(OdinCoreVer,pos('Core Files',OdinCoreVer)+11),'.','\')
            OdinSysVer   = translate(substr(OdinSysVer,pos('System Files',OdinSysVer)+13),'.','\')
            LastWarpIN = OdinSysDate' at 'OdinSysTime', it was  version 'OdinSysVer
        end
    end
return

/*:VRX         _CheckForEvent
*/
_CheckForEvent: 
    /* check and process event queue */
    EventString = ''
    do until EventString = 'nop'
        EventString = VREvent('N')
        if EventString <> 'nop' then do
            interpret EventString
        end
    end
    /* if UseRxNap then ok = RxNap(1) */
return

/*:VRX         _CheckForExpiredBuild
*/
_CheckForExpiredBuild: 
    if BuildFile = '' then return 0
    AllowReport = 0
    buildYear  = substr(BuildFile,11,4)
    buildmonth = substr(BuildFile,15,2)
    buildday   = substr(BuildFile,17,2)
    builddate  = substr(BuildFile,11,8)

    todayYear  = left(date('S'),4)
    todayMonth = substr(date('S'),5,2)
    todayDay   = right(date('S'),2)

    if datatype(buildyear) <> 'NUM' |datatype(buildmonth) <> 'NUM' |datatype(buildday) <> 'NUM' then do
        Msg.Text = "Invalid buildfilename! - Please enter a valid buildfilename!"
        Msg.Type = "Warning"
        call _ShowMsg
        return 0
    end
    datenumber  = BuildYear*365+BuildMonth*30+BuildDay
    todaynumber = TodayYear*365+TodayMonth*30+TodayDay
    select 
        when todaynumber - datenumber >  TooOld then do
            Msg.Text = "You have to try a newer build before submitting this bug report! Your bug might have been fixed already !"
            Msg.Type = "Error"
            call _ShowMsg
            if OverRideTooOld then do
                AllowReport = 1
                AgeOfBuild  = todaynumber - datenumber
                TooOldMsg   = 1
            end
        end
        when todaynumber - datenumber > WarnOld then do
            Msg.Text = "Before submitting this bug report, please try a newer build! Your bug might have been fixed already!"
            Msg.Type = "Warning"
            call _ShowMsg
            AllowReport = 1
        end
        otherwise AllowReport = 1
    end
return AllowReport
/*:VRX         _CheckInvalidreport
*/
_CheckInvalidreport: 
    Invalidreport = 0
    InvalidReport = InvalidReport + (VRGet("EF_Build","value") = "")
    InvalidReport = InvalidReport + (VRGet("DDCB_InstallType","value") = "")
    InvalidReport = InvalidReport + (VRGet("DDCB_LaunchType","value") = "")
    InvalidReport = InvalidReport + (VRGet("DDCB_OS2Version","value") = "")
    InvalidReport = InvalidReport + (VRGet("DDCB_FPK","value") = "")
    InvalidReport = InvalidReport + (VRGet("DDCB_DD","value") = "")
    InvalidReport = InvalidReport + (VRGet("EF_LogFile","value") = "")
    InvalidReport = InvalidReport + (VRGet("EF_EXEFile","value") = "")
    InvalidReport = InvalidReport + (VRGet("EF_Appname","value") = "")
    InvalidReport = InvalidReport + (VRGet("EF_OdinWinDir","value") = "")
    InvalidReport = InvalidReport + (VRGet("EF_OdinWinSysDir","value") = "")
    InvalidReport = InvalidReport + (VRGet("MLE_OwnWords","value") = "")
    InvalidReport = InvalidReport + (VRGet("EF_LastWarpIN","value") = "")
    InvalidReport = InvalidReport + (VRGet("EF_LastOdinInst","value") = "")
    if \ExceptionSearchDone then do
        ok = SysFileTree(ExceptFileName, ExceptDone., 'FO')
        if ExceptDone.0 = 0 then InvalidReport = 1
        else do
            ExceptionSearchDone = 1
            call _FillPageException
            ok = VRSet("EF_LogFile","Enabled", 1)
            ok = VRSet("PB_SearchLogFile","Enabled", 1)
        end
    end
    if AllowBugReport = 0 & OverRideTooOld = 0 then InvalidReport = 1
return sign(InvalidReport)

/*:VRX         _CommandLineParse
*/
_CommandLineParse: 
    if TestAppDir <> CurDirectory then do
        ok = SysFileTree(TestAppDir'Odin32_?.LOG',TestAppDirLogFile.,'FO')
        ok = SysFileTree(CurDirectory'Odin32_?.LOG',CurDirectoryLogFile.,'FO')
        if CurDirectoryLogFile.0 > 0 & TestAppDirLogFile.0 > 0 then do
            Msg.Type = "W"
            Msg.Text = "Logfiles found in "CurDirectory" and "TestAppDir". Be sure to choose the right one!"
            call _ShowMsg
        end
    end
    if VRGet("Application","Commandline") <> '' then do
        ok = SysFileTree(strip(VRGet("Application","Commandline"),'B','"'),Odin32Log., 'FO')
        if Odin32Log.0 = 1 then do
            LogFilename = Odin32Log.1
            TestAppDir  = strip(VRParseFileName( Odin32Log.1, "DP" ),'T','\') || '\'
        end
        else do
            Msg.Text = 'Unrecognized parameter ("'||VRGet("Application","Commandline")||'") - ignored!'
            Msg.Type = "Warning"
            call _ShowMsg
        end
    end
return

/*:VRX         _CreateExceptionAndStackDump
*/
_CreateExceptionAndStackDump: 
    call lineout RptFileName, 'Exception information and stack dump:'
    call lineout RptFileName, '  '
    do until lines(ExceptFileName) = 0
        call lineout RptFileName, linein(ExceptFileName)
    end
    ok = lineout(ExceptFileName)
return    

/*
    call lineout RptFileName, 'Exception information and stack dump:'
    call lineout RptFileName, '  '
    LogFileName = VRget("EF_LogFile","Value")
    if LogFileName = '' then return
    Buffer.0 = VRGet("SPIN_BufferSize","Value")
    LogFileSize = stream(LogfileName,'c','QUERY SIZE')
    call _InitProgress
    ApproxLines = LogFileSize % 41
    
    I = 0
    do while pos('Exception Information',LogLine) = 0 & lines(LogFileName) <> 0
        LogLine = linein(LogFileName)
        I = I + 1
        idx = I // 10 + 1
        Buffer.idx = LogLine
        if I // 1000 = 0 then do
            call _CheckForEvent
            Progress = I * 100 % ApproxLines
            ok = VRSet("SL_Progress","Percentile",Progress)
        end
    end
    if lines(LogFileName) = 0 then do
        call lineout RptFileName, 'No exception found in logfile!'
    end
    else do /* we had an exception lets create the dump */
        do J = idx+1 to 10
            call lineout RptFileName, Buffer.J
        end
        do J = 1 to idx
            call lineout RptFileName, Buffer.J
        end
        LogLine = linein(LogFileName)
        do until pos('End Of Exception Information',LogLine) > 0 | lines(LogFileName) = 0
            LogLine = linein(LogFileName)
            call lineout RptFileName, LogLine
        end

        do while pos('** BEGIN STACK DUMP **',LogLine) = 0 & lines(LogFileName) <> 0
            LogLine = linein(LogFileName)
            I = I + 1
            Progress = I * 100 % ApproxLines
            ok = VRSet("SL_Progress","Percentile",Progress)
        end 
        call lineout RptFileName, LogLine
        do until pos('** END OF STACK DUMP **',LogLine) > 0 | lines(LogFileName) = 0
            LogLine = linein(LogFileName)
            call lineout RptFileName,LogLine
        end
    end
    call lineout(LogFileName)
    ok = VRSet("Progress","Visible", 0) */

/*:VRX         _CreateObject
*/
_CreateObject: procedure
    Parse Arg Class, Title, Location, Setup, Collision
    /* Say 'Creating ['Title']' */
    rc = SysCreateObject( Class, Title, Location, Setup, Collision )
    If rc <> 1 Then do
        Msg.Text = ' > failed to create ['Title' | 'Class'] at location ['Location']'
        Msg.Type = 'Error'
        call _ShowMsg
    end
return rc

/*:VRX         _CreateReport
*/
_CreateReport: 
    RPTFileName = VRFileDialog( VRWindow(), "Save Odin Bug report to...", "Save", TestAppDir || "ODINBUG.RPT", , ,  )
    if RPTFilename = '' then return 0 /* not created */
    ok = SysFileTree(RPTFileName,Exists.,'FO')
    if Exists.0 = 1 then do
        Buttons.1 = "~Overwrite"
        Buttons.2 = "~Append"
        Buttons.0 = 2
        id = VRMessage( VRWindow(), RPTFileName||" exists!", "Odin Bug Report Generator", "Query", "Buttons.", buttons.1, buttons.2 )
        if id = 1 then do
            rc = SysFileDelete(RptFileName)
            if rc > 3 then do
                Msg.Type = "E"
                Msg.Text = "Could not open "RPTFileName" for writing! Aborting..."
                call _showMsg
                return 0 /* not created */
            end
        end
    end

    if VRGet("RB_OtherEmailClient","set") then call lineout RptFileName, '<sendto: 'EMail'>'
    call lineout RptFileName, '--------------------- Begin of Odin Bug Report ----------------------'
    if TooOldMsg then call lineout RptFileName, '                  for an expired build ('AgeOfBuild'days old)'
    call lineout RptFileName, '  '
    call lineout RptFileName, 'Odin build information:'
    call lineout RptFileName, '  '
    call lineout RptFileName, '  Current Odin build:     'VRGet("EF_Build","value")',  version 'Kernel32Ver
    call lineout RptFileName, '  Installation method:    'VRGet("DDCB_InstallType","value")
    call charout RptFileName, '  Launching method:       'VRGet("DDCB_LaunchType","value")
    if FoundWin32KSysNow & FoundWin32KSysAtBugTime then do
        call lineout RptFileName, ', Win32K-parameters: 'VRGet("EF_Win32Kparm","Value")
    end
    else call lineout RptFileName, '  '
    call lineout RptFileName, '  Last WarpIN install:    'VRGet("EF_LastWarpIN","Value")
    call lineout RptFileName, '  Last OdinInst install:  'VRGet("EF_LastOdinInst","Value")
    call lineout RptFileName, '  Odin "Windows" dir:     'VRget("EF_OdinWinDir","Value")
    call lineout RptFileName, '  Odin "System32" dir:    'VRget("EF_OdinWinSysDir","Value")
    call lineout RptFileName, '  Evaluated Odin.INI:     'VRGet("EF_OdinINI","Value")
    call lineout RptFileName, '  '
    call lineout RptFileName, 'General system information:'
    call lineout RptFileName, '  '
    call lineout RptFileName, '  OS/2 version:           'VRGet("DDCB_OS2Version","value")
    call lineout RptFileName, '  OS/2 reports version:   'VRGet("EF_Version","Value")', kernel revision: 'VRget("EF_KrnlBuildVer","Value")
    call lineout RptFileName, '  FixPack Level:          'VRGet("DDCB_FPK","value")
    call charout RptFileName, '  Is xf86sup.sys loaded?  '
    if Foundxf86supsys then call lineout RptFileName, 'Yes'
                       else call lineout RptFileName, 'No'
    call lineout RptFileName, '  Display driver:         'VRGet("DDCB_DD","value")' 'VRGet("DDCB_DDVer","Value")
    call lineout RptFileName, '  Video Chipset:          'VRGet("DDCB_chipset","Value")
    call lineout RptFileName, '  '
    call lineout RptFileName, 'Win32 program information:'
    call lineout RptFileName, '  '
    call lineout RptFileName, '  Odin logfile:           'VRget("EF_LogFile","Value")
    call lineout RptFileName, '  Windows executable:     'VRget("EF_EXEFile","Value")
    call lineout RptFileName, '  Application name:       'VRget("EF_AppName","Value")
    call lineout RptFileName, '  Application version:    'VRget("EF_AppVersion","Value")
    call lineout RptFileName, '  Available from:         'VRget("DDCB_URL","Value")
    call lineout RptFileName, '  '
    call lineout RptFileName, 'Verbal problem description:'
    call lineout RptFileName, '  '
    call lineout RptFileName, VRGet("MLE_OwnWords","Value")
    if OhNooo then do
        call lineout RptFileName, 'An "Oh, nooo!"-message box was displayed. '
    end
    call lineout RptFileName, '  '
    if AddException then call _CreateExceptionAndStackDump
                    else call lineout RptFileName, 'No exception information and stack dump added by user request!'
    call lineout RptFileName, '  '
    call lineout RptFileName, 'ODIN.INI information:'
    call lineout RptFileName, '  '
    do until lines(OdinIni) = 0
        call lineout RptFileName, linein(OdinIni)
    end
    call lineout OdinInI
    call lineout RptFileName, '---------------------- End of Odin Bug Report -----------------------'
    call lineout RptFileName, 'Created by OdinBug version 'word(VRGet("OdinBugReport","HintText"),2)' on 'date()' at 'time()
    call lineout RptFileName
return 1 /* success */

/*:VRX         _EMailGeneric
*/
_EMailGeneric: 
    if MessengerDir <> '' then do
        olddir = directory()
        call directory strip(MessengerDir,'T','\')
    end
    'start 'messenger' "mailto:'EMail'?subject=Odin Bug Report: 'VRget("EF_AppName","Value")' 'VRget("EF_AppVersion","Value")'"'  /* &body= */
    if MessengerDir <> '' then ok = directory(oldDir)
    CompleteReport = ''
    do until lines(RPTFileName) = 0
        CompleteReport = CompleteReport||CRLF||linein(RPTFileName)
    end
    call lineout RPTFileName
    ok = VRMethod("Application","PutClipBoard",CompleteReport)
    Msg.Text = "Report was copied to the clipboard! You may paste it into your email now!"
    Msg.Type = "Information"
    call _ShowMsg
return

/*:VRX         _EMailNetscape
*/
_EMailNetscape: 
    TempFile = strip(value('TEMP',,'OS2ENVIRONMENT'),'T','\')||'\OdinBug.$$$'
    ok = SysFileDelete(TempFile)
    ok = lineout(TempFile, 'To: 'EMail)
    ok = lineout(TempFile, 'Subject: Odin Bug Report: 'VRget("EF_AppName","Value")' 'VRget("EF_AppVersion","Value"))
    ok = lineout(TempFile, 'Content-Type: text/plain; charset=us-ascii')
    ok = lineout(TempFile, 'Content-Transfer-Encoding: 7bit')
    ok = charout(TempFile, CRLF)
    do until lines(RPTFileName) = 0
        ok = lineout(TempFile, linein(RPTFileName))
    end
    ok = Lineout(TempFile)
    ok = lineout(RPTFileName)
    if MessengerDir <> '' then do
        olddir = directory()
        call directory strip(MessengerDir,'T','\')
    end
    'start 'messenger' -compose 'TempFile
    if MessengerDir <> '' then ok = directory(oldDir)
return

/*:VRX         _EMailPMMSend
*/
_EMailPMMSend: 
    if MessengerDir <> '' then do
        olddir = directory()
        call directory strip(MessengerDir,'T','\')
    end
    ok = SysfileTree(MessengerDir'TEMP',TempDir., 'DO')
    if TempDir.0 = 0 then call _LaunchPMMail
    ok = SysfileTree(MessengerDir'TEMP',TempDir., 'DO')
    if TempDir.0 = 0 then do
        Msg.Type = 'Warning'
        Msg.Text = 'Failed to start PMMail! Please, check your email settings!'
        call _ShowMsg
    end
    'start /MIN /C 'messenger' /M "'RPTFileName'" "'left(EMail,pos('@',EMail)-1)' <'EMail'>" "Odin Bug Report: 'VRget("EF_AppName","Value")' 'VRget("EF_AppVersion","Value")'" "'VRGet("EF_Account","Value")'"'
    if MessengerDir <> '' then ok = directory(oldDir)
return

/*

*/

/* PMMSend Syntax:

USAGE: PMMSend <OPTION> <File To Send> <To E-Mail Address> <Subject>
       <Account Directory To Send From> <body file (/a option only)>

    Where <OPTION> is one of the following:

        /a Attachment option.  In this case the file to send will be an
                               attachment.  Note that the attachment type
                               will be read from the account config.
                               When using this option, you may specify
                               a text file to be used as the body of the
                               message.
        /f FAX option.  In this case the file to send will be a tiff
                        or other data file.
        /m Message option.  In this case the file to send will be a
                            message body.

EXAMPLE: PMMSEND /F c:\mydir\fax.tif "User <user@domain.com>"
                "A Fax For You" USER_D1.ACT

Options Must Begin With A Slash '/'  */
/*:VRX         _FillPageException
*/
_FillPageException: 
    ok = VRMethod( "LB_Exception", "Reset" ) 
    I = 1
    do until lines(ExceptFilename) = 0
         ExcLine.I = linein(ExceptFileName)
         I = I + 1
    end    
    ExcLine.0 = I
    ok = stream(ExceptFilename,"c", "close")
    ok = VRMethod("LB_Exception", "AddStringList", "ExcLine.") 
return

/*:VRX         _GetException
*/
_GetException: 
    threadID = VRMethod( "Application", "StartThread", "GetExcpt", LogFileName)
return

/*:VRX         _GotNewLogfileName
*/
_GotNewLogfileName: 
    ok = VRset("EF_LogFile","Value",LogFilename)
    TestAppDir  = strip(VRParseFileName( LogFileName, "DP" ),'T','\') || '\'
    ok = VRset("EF_TestAppDir","Value",TestAppDir)
    ok = SysFileDelete(ExceptFileName)
    ExceptionSearchDone = 0
    ok = VRSet("EF_LogFile","Enabled", 0)
    ok = VRSet("PB_SearchLogFile","Enabled", 0)
    ok = VRMethod("LB_Exception","Reset")

    call _AnalyseOdin32_xLog
    call _AnalyseKernel32DLL
    call _GUIRedraw2
    call _GetException
    ok = VRSet("PB_BrowseLogFile","Enabled",1)
    ok = VRSet("TM_CheckValid","Enabled",1)
return

/*:VRX         _GUIEMailInit
*/
_GUIEMailInit: 
    select
        when Messenger = 'NETSCAPE.EXE' then do
            ok = VRSEt("RB_Messenger461","set",1)
            if MessengerDir = "" then do
                ok = VRset("DT_MessengerDir","Visible", 0)
                ok = VRset("EF_MessengerDir","Visible", 0)
            end
            else do
                ok = VRset("DT_MessengerDir","Visible", 1)
                ok = VRset("EF_MessengerDir","Visible", 1)
            end
            ok = VRset("DDCB_Messenger","Visible", 0)
            ok = VRset("DT_Messenger","Visible", 0)
            ok = VRset("EF_Account","Visible", 0)
            ok = VRset("DT_Account","Visible", 0)
        end
        when Messenger = 'PMMSEND.EXE' then do
            ok = VRSEt("RB_PMMail","set",1)
            ok = VRset("DDCB_Messenger","Visible", 0)
            ok = VRset("EF_MessengerDir","Visible", 1)
            ok = VRset("EF_Account","Visible", 1)
            ok = VRset("DT_Messenger","Visible", 0)
            ok = VRset("DT_MessengerDir","Visible", 1)
            ok = VRset("DT_Account","Visible", 1)
        end
        otherwise do
            ok = VRSEt("RB_OtherEmailClient","set",1)
            ok = VRset("DDCB_Messenger","Visible", 1)
            ok = VRset("EF_MessengerDir","Visible", 1)
            ok = VRset("EF_Account","Visible", 0)
            ok = VRset("DT_Messenger","Visible", 1)
            ok = VRset("DT_MessengerDir","Visible", 1)
            ok = VRset("DT_Account","Visible", 0)
        end 
    end
return

/*:VRX         _GUIInit
*/
_GUIInit: 
    /* Odin Page */
    if WIN32LOG_ENABLED \= '' then ok = VRSet("CB_WIN32LOG_ENABLED","set",1)
                              else ok = VRSet("CB_WIN32LOG_ENABLED","set",0)

    ok = VRSet( "GB_Settings",       "HintText", "Currently using "our_ini)
    ok = VRSet( "EF_BuildDir",       "Value", BuildDir       )
    ok = VRSet( "EF_TestAppDir",     "Value", TestAppDir     )
    ok = VRSet( "EF_LogFileBrowser", "Value", LogFileBrowser )
    ok = VRSet( "SPIN_BufferSize",   "Value", BufferSize     )
    ok = VRSet( "CB_AddException",   "set",   AddException   )
    ok = VRSet( "CB_DontSaveAppDir", "set",   DontSaveAppDir )
    ok = VRSet( "CB_AskDeleteLog",   "set",   AskDeleteLog   )
    ok = VRset( "CB_xf86supsys",     "set",   0)

    /* EMail Page */

    ok = VRSet( "DDCB_EMail",        "Value", EMail          )
    ok = VRSet( "DDCB_Messenger",    "Value", Messenger      )
    ok = VRSet( "EF_MessengerDir",   "Value", MessengerDir   )
    ok = VRSet( "EF_Account",        "Value", Account        )

    call _GUIEMailInit

    /*  enter help file */
    ok = VRSet("OdinBugReport",   "HelpFile", base_dir"OdinBug.HLP")
    ok = VRSet("PageApplication", "HelpFile", base_dir"OdinBug.HLP")
    ok = VRSet("PageOdin",        "HelpFile", base_dir"OdinBug.HLP")
    ok = VRSet("PageSystem",      "HelpFile", base_dir"OdinBug.HLP")
    ok = VRSet("PageSettings",    "HelpFile", base_dir"OdinBug.HLP")
    ok = VRSet("PageEMail",       "HelpFile", base_dir"OdinBug.HLP")
return

/*:VRX         _GUIRedraw
*/
_GUIRedraw: 
    ok = VRset("CB_xf86supsys","set", Foundxf86supsys)
    if FoundWin32KSysNow then ok = VRset("DDCB_LaunchType","Value", "Win32K.SYS")
        else ok = VRSet("EF_Win32KParm","Value","Win32K.SYS is currently not installed!")
    ok = VRset("DDCB_DD","Value", Driver)
/*  ok = VRSet("DDCB_DDVer","Value","unknown") */
    ok = VRSet("DDCB_OS2Version","value", OS2Type)
    ok = VRSet("EF_KrnlBuildVer","value", KrnlBuildVer)
    ok = VRSet("EF_Version","Value",os_ver)
    ok = VRSet("DDCB_FPK","Value",fpk_level)
    if FoundScitech then do
        Ok = VRSet("DDCB_DDVer","Value",SddBuildNr)
        ok = VRSet("DDCB_ChipSet","value",Chipset)
    end
    ok = VRSet("EF_LastWarpIN","Value",LastWarpIN)
return

/*:VRX         _GUIRedraw2
*/
_GUIRedraw2: 
    if \FoundWin32KSysNow & FoundWin32KSysAtBugTime then do
        ok = VRset("DDCB_LaunchType","Value", "Win32K.SYS was installed, when the bug occured, but it is not now!")
    end
    if FoundWin32KSysNow & \FoundWin32KSysAtBugTime then do
        ok = VRset("DDCB_LaunchType","Value", "Win32K.SYS is installed now, but was not, when the bug occured!")
    end
    if \FoundWin32KSysNow & \FoundWin32KSysAtBugTime then do
        ok = VRset("DDCB_LaunchType","Value", "PE.EXE")
    end
    if FoundWin32KSysNow & FoundWin32KSysAtBugTime then do
        ok = VRSet("EF_Win32KParm","value",Win32Kparm)
    end
    ok = VRset("EF_OdinWindir","value",OdinWinDir)
    ok = VRset("EF_OdinWinSysdir","value",OdinSysDir)
    ok = VRset("EF_EXEFile","value",EXEFile)
    ok = VRset("EF_OdinINI","value",OdinINI)
    ok = VRMethod("EF_Appname","SetFocus")
return

/*:VRX         _INIRead
*/
_INIRead: /* Initialize variables, read values from INI file */

    /* delete old and obsolete files */
    ok = SysFileDelete('OdinBug.err')
    ok = SysFileDelete('OdinBug.CF')
    ok = SysFileDelete('OdinBug.CFG')

    /* Disable obsolete RxNap support */
    UseRxNap                = 0 /* \RxFuncQuery(RxNap) */
 
    /* Msg.Title = default title for message boxes */
    Msg.Title               = 'Odin Bug Report Generator'

    /*  Initialize variables - former undocumented cfg switches */
    WarnOld                 = 10 /* days */
    TooOld                  = 20 /* days */
    OverrideTooOld          = 0
    TooOldMsg               = 0
    AgeOfBuild              = 0
    AllowBugReport          = 0
    TempFile                = ''
    ExceptionSearchDone     = 0

    /* for abbreviation purposes */    
    CRLF                    = d2c(13)||d2c(10)

    /* ConfigSys vars */
    FoundWin32KSysNow       = 0
    Win32KConfigSys         = ''
    FoundScitech            = 0
    Foundxf86supsys         = 0
    Driver                  = ''

    /* odin32_?.log vars */
    LogFileName             = ''
    FoundWinDir             = 0
    FoundSysDir             = 0
    FoundExe                = 0
    FoundOdinIni            = 0
    FoundWin32KSysAtBugTime = 0
    EXEFile                 = ''
    OdinSysDir              = ''
    OdinWinDir              = ''
    OhNooo                  = 0

    /* Odin.INI */
    OdinInstLine            = '(not found)'
    OdinInstDate            = '(?)'
    OdinInstTime            = '(?)'
    LastOdinInst            = ''
    InstallType             = ''
    OdinINI                 = ''
    buildDate               = ''

    /* OdinInst.EXE */
    OdinInstVer             = '(unknown)'

    /* Kernel32.DLL */
    Kernel32Ver             = '(unknown)'
    Kernel32Date            = ''

    /* Scitech driver */
    SddBuildNr              = '(unknown)'

    /* OS/2 type */
    KrnlBuildVer            = '(unknown)'

    /* WarpIN stuff */
    LastWarpIN              = ''
    WarpINDB                = ''

    /* save current directory */
    CurDirectory            = strip(directory(),'T','\') || '\'

    /* Get logging status for current session - read only */ 
    win32log_enabled        = Value('WIN32LOG_ENABLED',,'OS2ENVIRONMENT')

    /* locate the base directory for INIfile */
    ok = SysFileTree('OdinBug.EXE',exe_file.,'FO')
    if exe_file.0 = 1 then exe_file = exe_file.1
                      else exe_file = SysSearchPath('PATH','OdinBug.EXE')

    /* This does not seem to be reliable...
    exe_file                = VRGet("Application","Program") 
     */
    base_dir                = strip(VRParseFileName( exe_file, "DP" ),'T','\') || '\'
    our_ini                 = base_dir || 'OdinBug.INI'

    /* read settings from our_ini */
    BuildDir                = VRGetIni( "OdinBug", "BuildDir", our_ini, 'NoClose' )
    TestAppDir              = VRGetIni( "OdinBug", "TestAppDir", our_ini, 'NoClose' )
    LogFileBrowser          = VRGetIni( "OdinBug", "LogFileBrowser", our_ini, 'NoClose' )
    DontSaveAppDir          = VRGetIni( "OdinBug", "DontSaveAppDir", our_ini, 'NoClose' )
    AskDeleteLog            = VRGetIni( "OdinBug", "AskDeleteLog", our_ini, 'NoClose' )
    EMail                   = VRGetIni( "OdinBug", "EMail", our_ini, 'NoClose' )
    Messenger     = translate(VRGetIni( "OdinBug", "Messenger", our_ini, 'NoClose' ))
    MessengerDir            = VRGetIni( "OdinBug", "MessengerDir", our_ini, 'NoClose' )
    Account                 = VRGetIni( "OdinBug", "Account", our_ini, 'NoClose' )
    BufferSize              = VRGetIni( "OdinBug", "BufferSize", our_ini, 'NoClose' )
    AddException            = VRGetIni( "OdinBug", "AddException", our_ini, 'NoClose' )
    OdinBugVersion          = VRGetIni( "OdinBug", "Version", our_ini )


    /*  new version ?  Update WPS objects! */
    if OdinBugVersion = '' | OdinBugVersion < word(VRGet("OdinBugReport","HintText"),2) then do
        call _InstallWPSObjects
        call PB_Help_Click
        OdinBugVersion = word(VRGet("OdinBugReport","HintText"),2)
        ok = VRSetIni( "OdinBug", "Version", OdinBugVersion , our_ini )
    end

    /* set default values */
    if BuildDir       = '' then Builddir = strip(directory(),'T','\')||'\'
    if DontSaveAppDir = '' then DontSaveAppDir = 1
    if AskDeleteLog   = '' then AskDeleteLog   = 0
    if TestAppDir     = '' then TestAppDir = strip(directory(),'T','\')||'\'
    if EMail          = '' then EMail = 'odinusers@yahoogroups.com'
    if Messenger      = '' then Messenger = 'NETSCAPE.EXE'
    /* MessengerDir   stays empty if empty */ 
    /* Account        stays empty if empty */
    /* LogFileBrowser stays empty if empty */
    if BufferSize     = '' then BufferSize = 10
    if AddException   = '' then AddException = 1

    /* Now let us process the commandline */    
    call _CommandLineParse
return

/*:VRX         _InitProgress
*/
_InitProgress: /* Initialize the progress bar */
    ok = VRSet("Progress","Visible", 1)
    ok = VRSet("DT_Progress","Visible", 0)
    ok = VRSet("SL_Progress","Percentile",0)
return

/*:VRX         _INIWrite
*/
_INIWrite: /* Write settings to INI file */
    ok = VRSetIni( "OdinBug", "BuildDir", BuildDir, our_ini, 'NoClose' )
    ok = VRSetIni( "OdinBug", "DontSaveAppDir", DontSaveAppDir, our_ini, 'NoClose' )
    if \DontSaveAppDir then ok = VRSetIni( "OdinBug", "TestAppDir", TestAppDir, our_ini, 'NoClose' )
    ok = VRSetIni( "OdinBug", "AskDeleteLog", AskDeleteLog, our_ini, 'NoClose' )
    ok = VRSetIni( "OdinBug", "EMail", EMail, our_ini, 'NoClose' )
    ok = VRSetIni( "OdinBug", "Messenger", translate(Messenger), our_ini, 'NoClose' )
    ok = VRSetIni( "OdinBug", "MessengerDir", MessengerDir, our_ini, 'NoClose' )
    ok = VRSetIni( "OdinBug", "LogFileBrowser", LogFileBrowser, our_ini, 'NoClose' )
    ok = VRSetIni( "OdinBug", "Account", Account, our_ini, 'NoClose' )
    ok = VRSetIni( "OdinBug", "BufferSize", BufferSize, our_ini, 'NoClose' )
    ok = VRSetIni( "OdinBug", "AddException", AddException, our_ini, 'NoClose' )
    ok = VRSetIni( "OdinBug", "Version", OdinBugVersion , our_ini )
return

/*:VRX         _InstallWPSObjects
*/
_InstallWPSObjects: 
    /* destroy old obsolete OdinBug object */
    ok = SysDestroyObject("<OdinBug>")

    /* Set Creation Disposition - we are polite and always update */
    CreateCollision = 'Update'

    rc1 = _CreateObject( 'WPFolder','Odin','<WP_PROGRAMSFOLDER>',,
            'HIDEBUTTON=DEFAULT;'||,
            'MINWIN=DEFAULT;'||,
            'CCVIEW=DEFAULT;'||,
            'DEFAULTVIEW=DEFAULT;'||,
            'OBJECTID=<ODINFOLDER>',,
        CreateCollision )

    rc2 = _CreateObject( 'WPProgram','OdinBug','<ODINFOLDER>',,
            'EXENAME='||exe_file||';'||,
            'PROGTYPE=PM;'||,
            'TITLE=Report Odin Bugs;'||,
            'ASSOCTYPE=Odin-Logfiles,,;'||,
            'ASSOCFILTER=odin32_?.log,,;'||,
            'NOPRINT=YES;'||,
            'HIDEBUTTON=DEFAULT;'||,
            'MINWIN=DEFAULT;'||,
            'CCVIEW=DEFAULT;'||,
            'DEFAULTVIEW=DEFAULT;'||,
            'OBJECTID=<Report_Odin_Bugs>',,
        CreateCollision )

    rc3 = _CreateObject( 'WPProgram','Report Odin Bugs Manual','<ODINFOLDER>',,
            'EXENAME=VIEW.EXE;'||,
            'PROGTYPE=PM;'||,
            'PARAMETERS='||base_dir||'ODINBUG.HLP Introduction;'||,
            'TITLE=Report Odin Bugs^Manual;'||,
            'NOPRINT=YES;'||,
            'HIDEBUTTON=DEFAULT;'||,
            'MINWIN=DEFAULT;'||,
            'CCVIEW=DEFAULT;'||,
            'DEFAULTVIEW=DEFAULT;'||,
            'OBJECTID=<Report_Odin_Bugs_Help>',,
        CreateCollision )
    rc4 = _CreateObject( 'WPProgram','Report Odin Bugs Readme','<ODINFOLDER>',,
            'EXENAME=E.EXE;'||,
            'PROGTYPE=PM;'||,
            'PARAMETERS='||base_dir||'Readme.ODINBUG;'||,
            'TITLE=Report Odin Bugs^ReadMe;'||,
            'NOPRINT=YES;'||,
            'HIDEBUTTON=DEFAULT;'||,
            'MINWIN=DEFAULT;'||,
            'CCVIEW=DEFAULT;'||,
            'DEFAULTVIEW=DEFAULT;'||,
            'OBJECTID=<Report_Odin_Bugs_ReadMe>',,
        CreateCollision )
    rc5 = _CreateObject( 'WPUrl','Subscribe to odinusers mailing list','<ODINFOLDER>',,
            'TITLE=Subscribe to^odinusers mailing list;'||,
            'URL=http://groups.yahoo.com/group/odinusers/join;'||,
            'HIDEBUTTON=DEFAULT;'||,
            'MINWIN=DEFAULT;'||,
            'CCVIEW=DEFAULT;'||,
            'DEFAULTVIEW=ICON;'||,
            'OBJECTID=<Subscribe_odinusers@yahoogroups>',,
        CreateCollision )

    if rc1 + rc2 + rc3 + rc4 + rc5 = 5 then do
        msg.Text = 'All OdinBug object(s) created/updated sucessfully!'
        msg.Type = 'Info'
        call _ShowMsg
    end
return
/*:VRX         _LaunchPMMail
*/
_LaunchPMMail: 
    ok = VRSEt("SL_Progress","Visible", 0)
    ok = VRSEt("DT_Progress","Visible", 1)
    ok = VRSet("DT_Progress","Caption", "Trying to start PMMail. Please wait...")
    ok = VRSet("Progress","Visible", 1)
    'start PMMAIL.EXE'
    ok = SysfileTree(MessengerDir'TEMP',TempDir., 'DO')
    Ctd = 10
    do while (TempDir.0 = 0) & (Ctd > 0)
        ok = SysSleep(1)
        Ctd = Ctd - 1
        ok = SysfileTree(MessengerDir'TEMP',TempDir., 'DO')
    end
    ok = VRSet("Progress","Visible", 0)
    ok = VRSEt("SL_Progress","Visible", 1)
    ok = VRSEt("DT_Progress","Visible", 0)
return

/*:VRX         _LoadOtherFuncs
*/
_LoadOtherFuncs: 
    /* Register RexxUtil-API */
    call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
    call SysLoadFuncs
return

/*:VRX         _ProcessHotKeys
*/
_ProcessHotKeys: 
    KeyObj = VRInfo( "Object" )
    KeyStr = VRGet( KeyObj, "KeyString" )
    select
        when KeyStr = "{Alt}c" then call PB_CreateReport_Click 
        when KeyStr = "{Alt}m" then call PB_Email_Click
        when KeyStr = "{Alt}o" then call PB_OpenReport_Click
        when KeyStr = "{Alt}b" then call PB_BrowseLogFile_Click 
        when KeyStr = "{Alt}h" then call PB_Help_Click 
        when KeyStr = "{Alt}x" then call PB_Exit_Click
        otherwise ok = VRSet(KeyObj, "KeyString", Keystr)
    end
return

/*:VRX         _ShowMsg
*/
_ShowMsg: 
    Buttons.1 = "OK"
    Buttons.0 = 1
    id = VRMessage( VRWindow(), Msg.Text, Msg.Title, Msg.Type, "Buttons.", buttons.1, buttons.1 )
return

/*:VRX         _StartUp
*/
_StartUp: 
    call _GUIInit
    call _AnalyseConfigSys
    call _AnalyseSysLevelOs2
    call _AnalyseSysLevelFPK
    if FoundScitech then call _AnalyseScitechDD
    call _AnalyseWarpINdatabase
    call _GUIRedraw

    if LogFileName = '' 
        then call PB_SearchLogFile_Click
        else call _GotNewLogfileName

    isStartup = 1
    call PB_SearchBuild_Click
return

/*:VRX         _xdate
*/
_xdate: procedure
    parse arg datestring
    hdate = ''
    do I = 1 to 2
        J = I * 2 - 1
        hdate = hdate||right('0'||x2d(substr(datestring,J,2)),2)'.'
    end
    Jahr = x2d(substr(datestring,5,2)) + x2d(substr(datestring,7,2)) *256
    hdate= hdate||Jahr
return hdate

/*:VRX         _xmonth
*/
_xmonth: procedure
   parse upper arg TextMonth
   select
       when left(TextMonth,3) = 'JAN' | left(TextMonth,3) = 'JN'then NumMonth = 1
       when left(TextMonth,3) = 'FEB' then NumMonth = 2
       when left(TextMonth,3) = 'MAR' | left(TextMonth,3) = 'MR'then NumMonth = 3
       when left(TextMonth,3) = 'APR' then NumMonth = 4
       when left(TextMonth,3) = 'MA'  then NumMonth = 5
       when left(TextMonth,3) = 'JUN' then NumMonth = 6
       when left(TextMonth,3) = 'JUL' then NumMonth = 7
       when left(TextMonth,3) = 'AUG' then NumMonth = 8
       when left(TextMonth,3) = 'SEP' then NumMonth = 9
       when left(TextMonth,3) = 'OCT' | left(TextMonth,3) = 'OKT'then NumMonth = 10
       when left(TextMonth,3) = 'NOV' then NumMonth = 11
       when left(TextMonth,2) = 'DE' then NumMonth = 12
       otherwise NumMonth = 0
   end
   NumMonth = right('0'||NumMonth,2)
return NumMonth

/*:VRX         _xtime
*/
_xtime: procedure
    parse arg timestring
    htime = ''
    do I = 1 to length(timestring)/2
        J = I * 2 - 1
        htime = htime||right('0'||x2d(substr(timestring,J,2)),2)':'
    end
    htime = strip(htime,'T',':')
return htime

/*:VRX         AboutPage_Close
*/
AboutPage_Close: 
    call AboutPage_Fini
return

/*:VRX         AboutPage_Create
*/
AboutPage_Create: 
    call AboutPage_Init
    ok = VRSet("DT_AboutText","Caption","OdinBug Version "word(VRget("OdinBugReport","HintText"),2)||CRLF||"(c) 2001 by Herwig Bauernfeind")
return

/*:VRX         AboutPage_Fini
*/
AboutPage_Fini: 
    window = VRInfo( "Window" )
    call VRDestroy window
    drop window
return
/*:VRX         AboutPage_Init
*/
AboutPage_Init: 
    window = VRInfo( "Object" )
    if( \VRIsChildOf( window, "Notebook" ) ) then do
        call VRMethod window, "CenterWindow"
        call VRSet window, "Visible", 1
        call VRMethod window, "Activate"
    end
    drop window
return

/*:VRX         CB_AddException_Click
*/
CB_AddException_Click: 
    AddException = VRget("CB_AddException","Set")
return

/*:VRX         CB_AskDeleteLog_Click
*/
CB_AskDeleteLog_Click: 
    AskDeleteLog = VRget("CB_AskDeleteLog","set")
return

/*:VRX         CB_DontSaveAppDir_Click
*/
CB_DontSaveAppDir_Click: 
    DontSaveAppDir = VRget("CB_DontSaveAppDir","set")
return

/*:VRX         CB_OhNooo_Click
*/
CB_OhNooo_Click: 
    OhNooo = VRGet("CB_OhNooo","set")
return

/*:VRX         CB_OverRideTooOld_Click
*/
CB_OverRideTooOld_Click: 
    OverRideTooOld = VRGet("CB_OverRideTooOld","set")
return

/*:VRX         CB_UseRxNap_Click
*/
CB_UseRxNap_Click: 
    UseRxNap then ok = VRGet("CB_UseRxNap","Set")
return

/*:VRX         CB_WIN32LOG_ENABLED_Click
*/
CB_WIN32LOG_ENABLED_Click: 
    if WIN32LOG_ENABLED \= '' then ok = VRSet("CB_WIN32LOG_ENABLED","set",1)
                              else ok = VRSet("CB_WIN32LOG_ENABLED","set",0)
return

/*:VRX         CB_xf86supsys_Click
*/
CB_xf86supsys_Click: 
    ok = VRSet("CB_xf86supsys","Set",Foundxf86supsys)
return

/*:VRX         DDCB_EMail_Change
*/
DDCB_EMail_Change: 
    if pos("@",VRGet("DDCB_EMail","Value")) = 0  then return
    EMail = VRGet("DDCB_EMail","Value")
return

/*:VRX         DDCB_LaunchType_Change
*/
DDCB_LaunchType_Change: 
    if FoundWin32KSysNow & FoundWin32KSysAtBugTime & VRGet("DDCB_LaunchType","value") = 'PE.EXE' then do
        Msg.Type = "W"
        Msg.Text = "Usage of PE.EXE is not recommended, if Win32K.SYS is installed!"
        call _ShowMsg
    end
return

/*:VRX         DDCB_Messenger_Change
*/
DDCB_Messenger_Change: 
    if pos(".EXE",translate(VRGet("DDCB_Messenger","Value"))) = 0  then return
    Messenger = translate(VRGet("DDCB_Messenger","Value"))
return

/*:VRX         EF_Account_Change
*/
EF_Account_Change: 
    Account = VRget("EF_Account","Value")
return

/*:VRX         EF_AppName_Change
*/
EF_AppName_Change: 
    if left(VRGet("EF_AppName","Value"),1) = '#' then do
        ok = VRset("EF_Appname","Value","Odin Bug Test")
        ok = VRset("EF_AppVersion","Value","9.9.9")
        ok = VRset("DDCB_URL","Value","www.odinbugtest.net")
        ok = VRSet("MLE_OwnWords","Value","This is test data. It is not intended to be sent to Team Odin support sites. The above website does not exist!")
    end
return

/*:VRX         EF_BuildDir_Change
*/
EF_BuildDir_Change: 
    BuildDir = VRGet("EF_BuildDir","Value")
    if Right(BuildDir,1) <> '\' then do
        BuildDir = BuildDir || '\'
        /* ok = VRSet("EF_BuildDir","Value", BuildDir) */
    end
return

/*:VRX         EF_LogFileBrowser_Change
*/
EF_LogFileBrowser_Change: 
    LogFileBrowser = VRget("EF_LogFileBrowser","Value")
return

/*:VRX         EF_MessengerDir_Change
*/
EF_MessengerDir_Change: 
    MessengerDir = VRget("EF_MessengerDir","Value")
    if Right(MessengerDir,1) <> '\' & MessengerDir <> '' then do
        MessengerDir = MessengerDir || '\'
        /* ok = VRSet("EF_MessengerDir","Value", MessengerDir) */
    end
return

/*:VRX         EF_OdinINI_Change
*/
EF_OdinINI_Change: 
    if right(VRGet("EF_OdinINI","Value"),8) <> 'ODIN.INI' then return
    ok = SysFileTree(VRGet("EF_OdinINI","Value"),OdinIniFile.,"FO")
    if OdinINiFile.0 = 0 then return
    OdinINI = VRGet("EF_OdinINI","Value")
    call _AnalyseOdininst
    call _AnalyseOdinINI
return

/*:VRX         EF_OdinWinSysDir_Change
*/
EF_OdinWinSysDir_Change: 
    if right(VRGet("EF_OdinWinSysDir","Value"),1) <> '\' then return
    ok = SysFileTree(strip(VRGet("EF_OdinWinSysDir","Value"),'T','\'),OdinWinSysDir.,"DO")
    if OdinWinSysDir.0 = 0 then return
    OdinSysDir = VRGet("EF_OdinWinSysDir","Value")
    call _AnalyseOdininst
return

/*:VRX         EF_TestAppDir_Change
*/
EF_TestAppDir_Change: 
    TestAppDir = VRget("EF_TestAppDir","Value")
    if Right(TestAppDir,1) <> '\' then do
        TestAppDir = TestAppDir || '\'
        /* ok = VRSet("EF_TestAppDir","Value", TestAppDir) */
    end
return

/*:VRX         Fini
*/
Fini:
    window = VRWindow()
    call VRSet window, "Visible", 0
    drop window
return 0

/*:VRX         Halt
*/
Halt:
    signal _VREHalt
return

/*:VRX         HelpWindow_Close
*/
HelpWindow_Close: 
    call HelpWindow_Fini
return

/*:VRX         HelpWindow_Create
*/
HelpWindow_Create: 
    call HelpWindow_Init
return

/*:VRX         HelpWindow_Fini
*/
HelpWindow_Fini: 
    window = VRInfo( "Window" )
    call VRDestroy window
    drop window
return
/*:VRX         HelpWindow_Init
*/
HelpWindow_Init: 
    window = VRInfo( "Object" )
    if( \VRIsChildOf( window, "Notebook" ) ) then do
        /* call VRMethod window, "CenterWindow" */
        call VRSet window, "Visible", 1
        call VRMethod window, "Activate"
    end
    drop window
return

/*:VRX         Init
*/
Init:
    window = VRWindow()
    call VRMethod window, "CenterWindow"
    call VRSet window, "Visible", 1
    call VRMethod window, "Activate"
    drop window
return

/*:VRX         NB_Main_PageSelected
*/
NB_Main_PageSelected: 
    /* CurPage = VRMethod("NB_Main","GetPageWindow",VRInfo("Page")) */
    CurPage = VRInfo("Page")
    select
        when CurPage = 1 then ok = VRMethod("EF_Appname","setfocus")
        when CurPage = 2 then ok = VRMethod("EF_Build","setfocus")
        when CurPage = 3 then ok = VRMethod("DDCB_OS2Version","Setfocus")
        when CurPage = 4 then do
            ok = VRSet("EF_BuildDir","Value", BuildDir)
            ok = VRSet("EF_TestAppDir","Value", TestAppDir)
            ok = VRMethod("EF_BuildDir","SetFocus")
        end
        when CurPage = 5 then do
            ok = VRSet("EF_MessengerDir","Value", MessengerDir)
            ok = VRMethod("DDCB_EMail","SetFocus")
        end
        otherwise nop
    end
return

/*:VRX         OdinBugReport_Close
*/
OdinBugReport_Close:
    if AskDeleteLog & LogFileName <> '' then do
        Buttons.1 = "Yes"
        Buttons.2 = "No"
        Buttons.0 = 2
        id = VRMessage( VRWindow(), 'Delete "'LogFileName'"?', "Odin Bug Report Generator", "Query", "Buttons.", 2, 2 )
        if id = 1 then ok = SysFileDelete(LogFileName)
    end
    call Quit
return

/*:VRX         OdinBugReport_Create
*/
OdinBugReport_Create: 
    call _LoadOtherFuncs
    window = VRLoadSecondary( "Progress" )
    call _INIRead
return

/*:VRX         PageApplication_Close
*/
PageApplication_Close: 
    call PageApplication_Fini
return

/*:VRX         PageApplication_Create
*/
PageApplication_Create: 
    call PageApplication_Init
    call _StartUp
return

/*:VRX         PageApplication_Fini
*/
PageApplication_Fini: 
    window = VRInfo( "Window" )
    call VRDestroy window
    drop window
return
/*:VRX         PageApplication_Init
*/
PageApplication_Init: 
    window = VRInfo( "Object" )
    if( \VRIsChildOf( window, "Notebook" ) ) then do
        call VRMethod window, "CenterWindow"
        call VRSet window, "Visible", 1
        call VRMethod window, "Activate"
    end
    drop window
return

/*:VRX         PageApplication_KeyPress
*/
PageApplication_KeyPress: 
    call _ProcessHotkeys
return

/*:VRX         PageEMail_Close
*/
PageEMail_Close: 
    call PageEMail_Fini
return

/*:VRX         PageEMail_Create
*/
PageEMail_Create: 
    call PageEMail_Init
return

/*:VRX         PageEMail_Fini
*/
PageEMail_Fini: 
    window = VRInfo( "Window" )
    call VRDestroy window
    drop window
return
/*:VRX         PageEMail_Init
*/
PageEMail_Init: 
    window = VRInfo( "Object" )
    if( \VRIsChildOf( window, "Notebook" ) ) then do
        call VRMethod window, "CenterWindow"
        call VRSet window, "Visible", 1
        call VRMethod window, "Activate"
    end
    drop window
return

/*:VRX         PageEMail_KeyPress
*/
PageEMail_KeyPress: 
    call _ProcessHotkeys
return

/*:VRX         PageException_Close
*/
PageException_Close: 
    call PageException_Fini
return

/*:VRX         PageException_Create
*/
PageException_Create: 
    call PageException_Init
return

/*:VRX         PageException_Fini
*/
PageException_Fini: 
    window = VRInfo( "Window" )
    call VRDestroy window
    drop window
return
/*:VRX         PageException_Init
*/
PageException_Init: 
    window = VRInfo( "Object" )
    if( \VRIsChildOf( window, "Notebook" ) ) then do
        call VRMethod window, "CenterWindow"
        call VRSet window, "Visible", 1
        call VRMethod window, "Activate"
    end
    drop window
return

/*:VRX         PageOdin_Close
*/
PageOdin_Close: 
    call PageOdin_Fini
return

/*:VRX         PageOdin_Create
*/
PageOdin_Create: 
    call PageOdin_Init
return

/*:VRX         PageOdin_Fini
*/
PageOdin_Fini: 
    window = VRInfo( "Window" )
    call VRDestroy window
    drop window
return
/*:VRX         PageOdin_Init
*/
PageOdin_Init: 
    window = VRInfo( "Object" )
    if( \VRIsChildOf( window, "Notebook" ) ) then do
        call VRMethod window, "CenterWindow"
        call VRSet window, "Visible", 1
        call VRMethod window, "Activate"
    end
    drop window
return

/*:VRX         PageOdin_KeyPress
*/
PageOdin_KeyPress: 
    call _ProcessHotkeys
return

/*:VRX         PageSettings_Close
*/
PageSettings_Close: 
    call PageSettings_Fini
return

/*:VRX         PageSettings_Create
*/
PageSettings_Create: 
    call PageSettings_Init
return

/*:VRX         PageSettings_Fini
*/
PageSettings_Fini: 
    window = VRInfo( "Window" )
    call VRDestroy window
    drop window
return
/*:VRX         PageSettings_Init
*/
PageSettings_Init: 
    window = VRInfo( "Object" )
    if( \VRIsChildOf( window, "Notebook" ) ) then do
        call VRMethod window, "CenterWindow"
        call VRSet window, "Visible", 1
        call VRMethod window, "Activate"
    end
    drop window
return

/*:VRX         PageSettings_KeyPress
*/
PageSettings_KeyPress: 
    call _ProcessHotKeys
return

/*:VRX         PageSystem_Close
*/
PageSystem_Close: 
    call PageSystem_Fini
return

/*:VRX         PageSystem_Create
*/
PageSystem_Create: 
    call PageSystem_Init
return

/*:VRX         PageSystem_Fini
*/
PageSystem_Fini: 
    window = VRInfo( "Window" )
    call VRDestroy window
    drop window
return
/*:VRX         PageSystem_Init
*/
PageSystem_Init: 
    window = VRInfo( "Object" )
    if( \VRIsChildOf( window, "Notebook" ) ) then do
        call VRMethod window, "CenterWindow"
        call VRSet window, "Visible", 1
        call VRMethod window, "Activate"
    end
    drop window
return

/*:VRX         PageSystem_KeyPress
*/
PageSystem_KeyPress: 
    call _ProcessHotkeys
return

/*:VRX         PB_About1_Click
*/
PB_About1_Click: 
    window = VRLoadSecondary( "AboutPage", "W" )
return

/*:VRX         PB_About_Click
*/
PB_About_Click: 
    window = VRLoadSecondary( "AboutPage", "W" )
return

/*:VRX         PB_AboutOK_Click
*/
PB_AboutOK_Click: 
    call AboutPage_Close
return

/*:VRX         PB_BrowseLogFile_Click
*/
PB_BrowseLogFile_Click: 
    'start "Browsing 'LogFileName'" /F 'LogFileBrowser' "'LogFileName'"'
return
/*:VRX         PB_CreateReport_Click
*/
PB_CreateReport_Click: 
    ok = time('R')
    ok = VRSet("TM_CheckValid","Enabled", 0)
    ok = VRSet("PB_CreateReport","Enabled", 0)
    if _CreateReport() then do
        ok = VRSet("SL_Progress","Percentile",100)
        Msg.Text = "Created "RPTFileName" successfully ("strip(time('E'),'T','0')" sec.)!"
        Msg.Type = "Info"
        call _ShowMsg    
        ok = VRSet("PB_Email","Enabled",1)
        ok = VRSet("PB_OpenReport","Enabled",1)
    end
    else ok = time('E')
    ok = VRSet("PB_CreateReport","Enabled", 1)
    ok = VRSet("TM_CheckValid","Enabled", 1)
return

/*:VRX         PB_Email_Click
*/
PB_Email_Click: 
    CapMessenger = translate(filespec('Name',Messenger))
    select
        when pos('NETSCAPE',CapMessenger) > 0 then call _EMailNetscape
        when pos('PMMSEND',CapMessenger) > 0  then call _EMailPMMSend
        otherwise call _EMailGeneric
    end
return

/*:VRX         PB_ExeFile_Click
*/
PB_ExeFile_Click: 
    ExeFile = VRFileDialog( VRWindow(), "Search Windows Executable", "Open", TestAppDir"*.EXE", , ,  )
    call _GUIRedraw2
return

/*:VRX         PB_Exit_Click
*/
PB_Exit_Click: 
    call OdinBugReport_Close
return
/*:VRX         PB_Help_Click
*/
PB_Help_Click: 
    'view 'base_dir"OdinBug.HLP Introduction"
    /* ok = SysSetObjectData(base_dir"OdinBug.HLP","OPEN=DEFAULT;PARAMETERS=Introduction") */
return
/*:VRX         PB_Install1_Click
*/
PB_Install1_Click: 
    call _InstallWPSObjects
return
/*:VRX         PB_Install_Click
*/
PB_Install_Click: 
    call _InstallWPSObjects
return
/*:VRX         PB_OpenReport_Click
*/
PB_OpenReport_Click: 
    ok = SysSetObjectData(RPTFileName,"OPEN=DEFAULT")
return
/*:VRX         PB_SaveSettings1_Click
*/
PB_SaveSettings1_Click: 
    call _INIWrite
    msg.Text = 'Settings saved sucessfully!'
    msg.Type = 'Info'
    call _ShowMsg
return

/*:VRX         PB_SaveSettings_Click
*/
PB_SaveSettings_Click: 
    call _INIWrite
    msg.Text = 'Settings saved sucessfully!'
    msg.Type = 'Info'
    call _ShowMsg
return

/*:VRX         PB_SearchBuild_Click
*/
PB_SearchBuild_Click: 
    ok = SysFileTree(BuildDir"odin32bin-"Kernel32Date"*",Builds.,"FO")
    if Builds.0 = 1 & isStartup
        then Filename = Builds.1
        else FileName = VRFileDialog( VRWindow(), "Step 2: Search Odin Build file", "Open", BuildDir"odin32bin-"Kernel32Date"*", , ,  )

    if Filename <> '' then do
        BuildFile = filespec('N',Filename)
        BuildDir = filespec('D',Filename)||filespec('P',Filename)
        if translate(right(Filename,3)) = 'WPI' then Installtype = "WarpIN"
        if translate(right(Filename,3)) = 'ZIP' then Installtype = "Manually"

        AllowBugReport = _CheckForExpiredBuild()
        ok = VRset("EF_Build","Value",BuildFile)
        ok = VRset("EF_BuildDir","Value",BuildDir)
        ok = VRset("DDCB_Installtype","Value",InstallType)
    end
    isStartup = 0
return

/*:VRX         PB_SearchLogFile_Click
*/
PB_SearchLogFile_Click: 
    LogFileName = VRFileDialog( VRWindow(), "Step 1: Search Odin Logfile", "Open", TestAppDir"odin32_?.log", , ,  )
    if LogFileName <> '' 
        then call _GotNewLogfileName
        else ok = VRSet("PB_BrowseLogFile","Enabled",0)
return

/*:VRX         PB_SearchLogFileBrowser_Click
*/
PB_SearchLogFileBrowser_Click: 
    LogFileBrowser = VRFileDialog( VRWindow(), "Search Logfile browser", "Open", "*.EXE", , ,  )
    if LogFileBrowser <> '' then ok = VRset("EF_LogFileBrowser","Value",LogFileBrowser)
return

/*:VRX         PB_ThankYou_Click
*/
PB_ThankYou_Click: 
    Lines.1 = "Many thanks go to Team Odin and their efforts, especially to Sander van Leeuwen for his (almost) endless patience and to my testers Sven Stroh and Peter Weilbacher"
    Lines.2 = " "
    Lines.3 = "This program is free, but copyrighted software. The license for Odin is also valid for this program, except for source code, which is currently available by request to the author."
    Lines.0 = 3
    Buttons.1 = "OK"
    Buttons.0 = 1
    id = VRMessageStem( VRWindow(), "Lines.", "Thank you", "Information", "Buttons.", buttons.1, buttons.1 )
return

/*:VRX         Progress_Close
*/
Progress_Close: 
    call Progress_Fini
return

/*:VRX         Progress_Create
*/
Progress_Create: 
    call Progress_Init
    ok = VRSet("Progress","Top",5000)
return

/*:VRX         Progress_Fini
*/
Progress_Fini: 
    window = VRInfo( "Window" )
    call VRDestroy window
    drop window
return
/*:VRX         Progress_Init
*/
Progress_Init: 
    window = VRInfo( "Object" )
    if( \VRIsChildOf( window, "Notebook" ) ) then do
        call VRMethod window, "CenterWindow"
        call VRSet window, "Visible", 0
        call VRMethod window, "Activate"
    end
    drop window
return

/*:VRX         Quit
*/
Quit:
    ok = SysFileDelete(ExceptFileName)
    window = VRWindow()
    call VRSet window, "Shutdown", 1
    drop window
return

/*:VRX         RB_Messenger461_Click
*/
RB_Messenger461_Click: 
    if VRGet("RB_Messenger461","Set") then do
        ok = VRset("DDCB_Messenger","Value",  'NETSCAPE.EXE')
        ok = VRset("EF_MessengerDir","Value", '')
        Messenger = 'NETSCAPE.EXE'
        CompleteMessenger = SysSearchPath("PATH",Messenger)
        if CompleteMessenger = '' then do
            Msg.Type = "W"
            Msg.Text = Messenger' could not be found in your PATH. Please, specify the EMail client directory!'
            call _ShowMsg
            ok = VRset("EF_MessengerDir","Visible", 1)
            ok = VRset("DT_MessengerDir","Visible", 1)
        end
        else do
            ok = VRset("EF_MessengerDir","Visible", 0)
            ok = VRset("DT_MessengerDir","Visible", 0)
        end
        ok = VRset("DDCB_Messenger","Visible", 0)
        ok = VRset("DT_Messenger","Visible", 0)
        ok = VRset("EF_Account",  "Visible", 0)
        ok = VRset("DT_Account",  "Visible", 0)
    end
return

/*:VRX         RB_OtherEmailClient_Click
*/
RB_OtherEmailClient_Click: 
    if VRget("RB_OtherEmailClient","set") then do
        ok = VRset("DDCB_Messenger","Visible", 1)
        ok = VRset("EF_MessengerDir","Visible", 1)
        ok = VRset("EF_Account","Visible", 0)
        ok = VRset("DT_Messenger","Visible", 1)
        ok = VRset("DT_MessengerDir","Visible", 1)
        ok = VRset("DT_Account","Visible", 0)
    end
return
/*:VRX         RB_PMMail_Click
*/
RB_PMMail_Click: 
    if VRGet("RB_PMMail","set") then do
        ok = VRset("DDCB_Messenger","Value", 'PMMSEND.EXE')
        Messenger = 'PMMSEND.EXE'
        CompleteMessenger = SysSearchPath("PATH",Messenger)
        if CompleteMessenger = '' then do
            Msg.Type = "W"
            Msg.Text = Messenger' could not be found in your PATH. Please, specify the EMail client directory manually!'
            call _ShowMsg
        end
        else do
            ok = VRset("EF_MessengerDir","Value", filespec("D",CompleteMessenger)||filespec("P",CompleteMessenger))
        end
        ok = VRset("DDCB_Messenger","Visible", 0)
        ok = VRset("DT_Messenger","Visible", 0)
        ok = VRset("EF_MessengerDir","Visible", 1)
        ok = VRset("DT_MessengerDir","Visible", 1)
        ok = VRset("EF_Account","Visible", 1)
        ok = VRset("DT_Account","Visible", 1)
    end
return

/*:VRX         SPIN_BufferSize_Change
*/
SPIN_BufferSize_Change: 
    BufferSize = VRget("SPIN_BufferSize","value")
return

/*:VRX         TM_CheckValid_Trigger
*/
TM_CheckValid_Trigger: 
    ok = VRSet("PB_CreateReport","Enabled",\_CheckInvalidReport())
return

